@@ -29,7 +29,7 @@ fn basic_cases() {
         });
 
         (0..cnt).map(|i| (i, gen_sample(i))).for_each(|(i, b)| {
-            hdr_i.entry_ref(&i).or_insert_ref(&b);
+            hdr_i.entry(i).or_insert(b.clone());
             assert_eq!(1 + i as usize, hdr_i.len());
             assert_eq!(pnk!(hdr_i.get(&i)).idx, i);
             assert_eq!(hdr_i.remove(&i), Some(b.clone()));
