@@ -9,7 +9,7 @@
 //! # Examples
 //!
 //! ```
-//! use vsdb::MapxOrdRawValue;
+//! use vsdb::basic::mapx_ord_rawvalue::MapxOrdRawValue;
 //!
 //! let mut l = MapxOrdRawValue::new();
 //!
@@ -35,41 +35,23 @@ mod test;
 
 use crate::{
     basic::mapx_raw::{MapxRaw, MapxRawIter},
-    common::{
-        ende::{KeyEnDeOrdered, SimpleVisitor, ValueEnDe},
-        InstanceCfg, RawValue,
-    },
+    common::{ende::KeyEnDeOrdered, RawValue},
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
 use std::{
     marker::PhantomData,
-    mem::ManuallyDrop,
     ops::{Bound, Deref, DerefMut, RangeBounds},
-    result::Result as StdResult,
 };
 
-/// To solve the problem of unlimited memory usage,
-/// use this to replace the original in-memory `BTreeMap<_, _>`.
-#[derive(PartialEq, Eq, Debug)]
+#[derive(Clone, Copy, Serialize, Deserialize, PartialEq, Eq, Debug)]
+#[serde(bound = "")]
 pub struct MapxOrdRawValue<K>
 where
     K: KeyEnDeOrdered,
 {
     inner: MapxRaw,
-    _pd: PhantomData<K>,
-}
-
-impl<K> From<InstanceCfg> for MapxOrdRawValue<K>
-where
-    K: KeyEnDeOrdered,
-{
-    fn from(cfg: InstanceCfg) -> Self {
-        Self {
-            inner: MapxRaw::from(cfg),
-            _pd: PhantomData,
-        }
-    }
+    p: PhantomData<K>,
 }
 
 impl<K> Default for MapxOrdRawValue<K>
@@ -81,34 +63,23 @@ where
     }
 }
 
-////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////
-
 impl<K> MapxOrdRawValue<K>
 where
     K: KeyEnDeOrdered,
 {
-    /// Create an instance.
     #[inline(always)]
     pub fn new() -> Self {
         MapxOrdRawValue {
             inner: MapxRaw::new(),
-            _pd: PhantomData,
+            p: PhantomData,
         }
     }
 
-    // Get the database storage path
-    pub(crate) fn get_instance_cfg(&self) -> InstanceCfg {
-        self.inner.get_instance_cfg()
-    }
-
-    /// Imitate the behavior of 'BTreeMap<_>.get(...)'
     #[inline(always)]
     pub fn get(&self, key: &K) -> Option<RawValue> {
         self.inner.get(&key.to_bytes())
     }
 
-    /// Get the closest smaller value, include itself.
     #[inline(always)]
     pub fn get_le(&self, key: &K) -> Option<(K, RawValue)> {
         self.inner
@@ -116,7 +87,6 @@ where
             .map(|(k, v)| (pnk!(K::from_bytes(k)), v))
     }
 
-    /// Get the closest larger value, include itself.
     #[inline(always)]
     pub fn get_ge(&self, key: &K) -> Option<(K, RawValue)> {
         self.inner
@@ -124,7 +94,6 @@ where
             .map(|(k, v)| (pnk!(K::from_bytes(k)), v))
     }
 
-    /// Imitate the behavior of 'BTreeMap<_>.get_mut(...)'
     #[inline(always)]
     pub fn get_mut(&mut self, key: &K) -> Option<ValueMut<'_, K>> {
         self.inner
@@ -132,70 +101,59 @@ where
             .map(|v| ValueMut::new(self, key.clone(), v))
     }
 
-    /// Imitate the behavior of 'BTreeMap<_>.len()'.
     #[inline(always)]
     pub fn len(&self) -> usize {
         self.inner.len()
     }
 
-    /// A helper func
     #[inline(always)]
     pub fn is_empty(&self) -> bool {
         self.inner.is_empty()
     }
 
-    /// Imitate the behavior of 'BTreeMap<_>.insert(...)'.
     #[inline(always)]
     pub fn insert(&mut self, key: K, value: RawValue) -> Option<RawValue> {
         self.insert_ref(&key, &value)
     }
 
     #[inline(always)]
-    #[allow(missing_docs)]
     pub fn insert_ref(&mut self, key: &K, value: &[u8]) -> Option<RawValue> {
         self.inner.insert(&key.to_bytes(), value)
     }
 
-    /// Similar with `insert`, but ignore the old value.
     #[inline(always)]
     pub fn set_value(&mut self, key: K, value: RawValue) {
         self.set_value_ref(&key, &value);
     }
 
     #[inline(always)]
-    #[allow(missing_docs)]
     pub fn set_value_ref(&mut self, key: &K, value: &[u8]) {
         self.inner.insert(&key.to_bytes(), value);
     }
 
-    /// Imitate the behavior of '.entry(...).or_insert(...)'
     #[inline(always)]
     pub fn entry(&mut self, key: K) -> Entry<'_, K> {
         Entry { key, hdr: self }
     }
 
     #[inline(always)]
-    #[allow(missing_docs)]
     pub fn entry_ref<'a>(&'a mut self, key: &'a K) -> EntryRef<'a, K> {
         EntryRef { key, hdr: self }
     }
 
-    /// Imitate the behavior of '.iter()'
     #[inline(always)]
     pub fn iter(&self) -> MapxOrdRawValueIter<K> {
         MapxOrdRawValueIter {
             iter: self.inner.iter(),
-            _pd: PhantomData,
+            p: PhantomData,
         }
     }
 
-    /// range(start..end)
     #[inline(always)]
     pub fn range<R: RangeBounds<K>>(&self, bounds: R) -> MapxOrdRawValueIter<K> {
         self.range_ref((bounds.start_bound(), bounds.end_bound()))
     }
 
-    /// range(start..end)
     #[inline(always)]
     pub fn range_ref<'a, R: RangeBounds<&'a K>>(
         &'a self,
@@ -229,59 +187,49 @@ where
 
         MapxOrdRawValueIter {
             iter: self.inner.range((l, h)),
-            _pd: PhantomData,
+            p: PhantomData,
         }
     }
 
-    /// First item
     #[inline(always)]
     pub fn first(&self) -> Option<(K, RawValue)> {
         self.iter().next()
     }
 
-    /// Last item
     #[inline(always)]
     pub fn last(&self) -> Option<(K, RawValue)> {
         self.iter().next_back()
     }
 
-    /// Check if a key is exists.
     #[inline(always)]
     pub fn contains_key(&self, key: &K) -> bool {
         self.inner.contains_key(&key.to_bytes())
     }
 
-    /// Remove a <K> from mem and disk.
     #[inline(always)]
     pub fn remove(&mut self, key: &K) -> Option<RawValue> {
         self.inner.remove(&key.to_bytes())
     }
 
-    /// Remove a <K> from mem and disk.
     #[inline(always)]
     pub fn unset_value(&mut self, key: &K) {
         self.inner.remove(&key.to_bytes());
     }
 
-    /// Clear all data.
     #[inline(always)]
     pub fn clear(&mut self) {
         self.inner.clear();
     }
 }
 
-////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////
-
-/// Returned by `<MapxOrdRawValue>.get_mut(...)`
 #[derive(Debug)]
 pub struct ValueMut<'a, K>
 where
     K: KeyEnDeOrdered,
 {
     hdr: &'a mut MapxOrdRawValue<K>,
-    key: ManuallyDrop<K>,
-    value: ManuallyDrop<RawValue>,
+    key: K,
+    value: RawValue,
 }
 
 impl<'a, K> ValueMut<'a, K>
@@ -289,28 +237,16 @@ where
     K: KeyEnDeOrdered,
 {
     pub(crate) fn new(hdr: &'a mut MapxOrdRawValue<K>, key: K, value: RawValue) -> Self {
-        ValueMut {
-            hdr,
-            key: ManuallyDrop::new(key),
-            value: ManuallyDrop::new(value),
-        }
+        ValueMut { hdr, key, value }
     }
 }
 
-/// NOTE: Very Important !!!
 impl<'a, K> Drop for ValueMut<'a, K>
 where
     K: KeyEnDeOrdered,
 {
     fn drop(&mut self) {
-        // This operation is safe within a `drop()`.
-        // SEE: [**ManuallyDrop::take**](std::mem::ManuallyDrop::take)
-        unsafe {
-            self.hdr.set_value(
-                ManuallyDrop::take(&mut self.key),
-                ManuallyDrop::take(&mut self.value),
-            );
-        };
+        self.hdr.set_value_ref(&self.key, &self.value);
     }
 }
 
@@ -334,10 +270,6 @@ where
     }
 }
 
-////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////
-
-/// Imitate the `btree_map/btree_map::Entry`.
 pub struct Entry<'a, K>
 where
     K: KeyEnDeOrdered,
@@ -350,7 +282,6 @@ impl<'a, K> Entry<'a, K>
 where
     K: KeyEnDeOrdered,
 {
-    /// Imitate the `btree_map/btree_map::Entry.or_insert(...)`.
     pub fn or_insert(self, default: RawValue) -> ValueMut<'a, K> {
         if !self.hdr.contains_key(&self.key) {
             self.hdr.set_value_ref(&self.key, &default);
@@ -359,7 +290,6 @@ where
     }
 }
 
-#[allow(missing_docs)]
 pub struct EntryRef<'a, K>
 where
     K: KeyEnDeOrdered,
@@ -372,7 +302,6 @@ impl<'a, K> EntryRef<'a, K>
 where
     K: KeyEnDeOrdered,
 {
-    /// Imitate the `btree_map/btree_map::Entry.or_insert(...)`.
     pub fn or_insert_ref(self, default: &[u8]) -> ValueMut<'a, K> {
         if !self.hdr.contains_key(self.key) {
             self.hdr.set_value_ref(self.key, default);
@@ -381,16 +310,12 @@ where
     }
 }
 
-////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////
-
-#[allow(missing_docs)]
 pub struct MapxOrdRawValueIter<K>
 where
     K: KeyEnDeOrdered,
 {
     iter: MapxRawIter,
-    _pd: PhantomData<K>,
+    p: PhantomData<K>,
 }
 
 impl<K> Iterator for MapxOrdRawValueIter<K>
@@ -415,37 +340,3 @@ where
 }
 
 impl<K> ExactSizeIterator for MapxOrdRawValueIter<K> where K: KeyEnDeOrdered {}
-
-////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////
-
-impl<K> Serialize for MapxOrdRawValue<K>
-where
-    K: KeyEnDeOrdered,
-{
-    fn serialize<S>(&self, serializer: S) -> StdResult<S::Ok, S::Error>
-    where
-        S: serde::Serializer,
-    {
-        serializer.serialize_bytes(&<InstanceCfg as ValueEnDe>::encode(
-            &self.get_instance_cfg(),
-        ))
-    }
-}
-
-impl<'de, K> Deserialize<'de> for MapxOrdRawValue<K>
-where
-    K: KeyEnDeOrdered,
-{
-    fn deserialize<D>(deserializer: D) -> StdResult<Self, D::Error>
-    where
-        D: serde::Deserializer<'de>,
-    {
-        deserializer.deserialize_bytes(SimpleVisitor).map(|cfg| {
-            MapxOrdRawValue::from(<InstanceCfg as ValueEnDe>::decode(&cfg).unwrap())
-        })
-    }
-}
-
-////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////
