@@ -8,16 +8,15 @@ pub(crate) mod ende;
 pub(crate) mod engines;
 
 use {
+    crc32fast::Hasher,
     engines::Engine,
-    lazy_static::lazy_static,
+    once_cell::sync::Lazy,
+    parking_lot::Mutex,
     ruc::*,
-    serde::{Deserialize, Serialize},
-    sha3::{Digest, Sha3_256},
     std::{
         env, fs,
         mem::size_of,
         sync::atomic::{AtomicBool, Ordering},
-        sync::{Arc, Mutex},
     },
 };
 
@@ -28,32 +27,49 @@ pub(crate) type RawBytes = Box<[u8]>;
 pub(crate) type RawKey = RawBytes;
 pub(crate) type RawValue = RawBytes;
 
+/// Checksum of a version
+pub type VerChecksum = [u8; size_of::<u32>()];
+
 pub(crate) type Prefix = u64;
 pub(crate) type PrefixBytes = [u8; PREFIX_SIZ];
 pub(crate) const PREFIX_SIZ: usize = size_of::<Prefix>();
 
-const RESERVED_ID_CNT: Prefix = 4096;
-pub(crate) const BIGGEST_RESERVED_ID: Prefix = RESERVED_ID_CNT - 1;
-
 pub(crate) type BranchID = u64;
 pub(crate) type VersionID = u64;
 
+/// avoid making mistakes between branch name and version name
+#[derive(Clone, Copy)]
+pub struct BranchName<'a>(pub &'a [u8]);
+/// +1
+#[derive(Clone, Copy)]
+pub struct ParentBranchName<'a>(pub &'a [u8]);
+/// +1
+#[derive(Clone, Copy)]
+pub struct VersionName<'a>(pub &'a [u8]);
+
+const RESERVED_ID_CNT: Prefix = 4096_0000;
+pub(crate) const BIGGEST_RESERVED_ID: Prefix = RESERVED_ID_CNT - 1;
+pub(crate) const NULL: BranchID = BIGGEST_RESERVED_ID;
+
+pub(crate) const INITIAL_BRANCH_ID: BranchID = 0;
+pub(crate) const INITIAL_BRANCH_NAME: &[u8] = b"main";
+
+/// how many branches in one instance can be created
+pub const BRANCH_CNT_LIMIT: usize = 1024;
+
+// default value for reserved number when pruning old data
+pub(crate) const RESERVED_VERSION_NUM_DEFAULT: usize = 10;
+
 /////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////
 
-lazy_static! {
-    static ref VSDB_BASE_DIR: Arc<Mutex<String>> = Arc::new(Mutex::new(gen_data_dir()));
-}
+static VSDB_BASE_DIR: Lazy<Mutex<String>> = Lazy::new(|| Mutex::new(gen_data_dir()));
 
 #[cfg(all(feature = "sled_engine", not(feature = "rocks_engine")))]
-lazy_static! {
-    pub(crate) static ref VSDB: VsDB<engines::Sled> = pnk!(VsDB::new());
-}
+pub(crate) static VSDB: Lazy<VsDB<engines::Sled>> = Lazy::new(|| pnk!(VsDB::new()));
 
 #[cfg(all(feature = "rocks_engine", not(feature = "sled_engine")))]
-lazy_static! {
-    pub(crate) static ref VSDB: VsDB<engines::RocksDB> = pnk!(VsDB::new());
-}
+pub(crate) static VSDB: Lazy<VsDB<engines::RocksDB>> = Lazy::new(|| pnk!(VsDB::new()));
 
 /////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////
@@ -62,7 +78,7 @@ lazy_static! {
 #[macro_export(crate)]
 macro_rules! parse_int {
     ($bytes: expr, $ty: ty) => {{
-        let array: [u8; std::mem::size_of::<$ty>()] = $bytes.try_into().unwrap();
+        let array: [u8; std::mem::size_of::<$ty>()] = $bytes[..].try_into().unwrap();
         <$ty>::from_be_bytes(array)
     }};
 }
@@ -83,20 +99,24 @@ pub(crate) struct VsDB<T: Engine> {
 }
 
 impl<T: Engine> VsDB<T> {
+    #[inline(always)]
     fn new() -> Result<Self> {
         Ok(Self {
             db: T::new().c(d!())?,
         })
     }
 
+    #[inline(always)]
     pub(crate) fn alloc_branch_id(&self) -> BranchID {
         self.db.alloc_branch_id()
     }
 
+    #[inline(always)]
     pub(crate) fn alloc_version_id(&self) -> VersionID {
         self.db.alloc_version_id()
     }
 
+    #[inline(always)]
     fn flush(&self) {
         self.db.flush()
     }
@@ -115,19 +135,17 @@ fn gen_data_dir() -> String {
 }
 
 fn get_data_dir() -> String {
-    VSDB_BASE_DIR.lock().unwrap().clone()
+    VSDB_BASE_DIR.lock().clone()
 }
 
 /// Set ${VSDB_BASE_DIR} manually
 pub fn vsdb_set_base_dir(dir: String) -> Result<()> {
-    lazy_static! {
-        static ref HAS_INITED: AtomicBool = AtomicBool::new(false);
-    }
+    static HAS_INITED: AtomicBool = AtomicBool::new(false);
 
     if HAS_INITED.swap(true, Ordering::Relaxed) {
         Err(eg!("VSDB has been initialized !!"))
     } else {
-        *VSDB_BASE_DIR.lock().unwrap() = dir;
+        *VSDB_BASE_DIR.lock() = dir;
         Ok(())
     }
 }
@@ -140,22 +158,13 @@ pub fn vsdb_flush() {
 /////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////
 
-#[derive(Deserialize, Serialize, Debug)]
-pub(crate) struct InstanceCfg {
-    pub(crate) prefix: PrefixBytes,
-    pub(crate) item_cnt: u64,
-    pub(crate) area_idx: usize,
-}
-
-/////////////////////////////////////////////////////////////////////////////
-/////////////////////////////////////////////////////////////////////////////
-
-pub(crate) fn compute_sig(ivec: &[&[u8]]) -> RawBytes {
-    let mut hasher = Sha3_256::new();
+#[inline(always)]
+pub(crate) fn compute_checksum(ivec: &[&[u8]]) -> VerChecksum {
+    let mut hasher = Hasher::new();
     for bytes in ivec {
         hasher.update(bytes);
     }
-    hasher.finalize().as_slice().to_vec().into_boxed_slice()
+    hasher.finalize().to_be_bytes()
 }
 
 /////////////////////////////////////////////////////////////////////////////
