@@ -31,7 +31,6 @@
 //! Stateless functions do not have the feature of 'version' management, but they have higher performance.
 
 #![deny(warnings)]
-#![deny(missing_docs)]
 #![recursion_limit = "512"]
 
 pub mod basic;
@@ -40,18 +39,16 @@ pub mod versioned;
 
 pub use basic::mapx::Mapx;
 pub use basic::mapx_ord::MapxOrd;
-pub use basic::mapx_ord_rawkey::MapxOrdRawKey;
-pub use basic::mapx_ord_rawvalue::MapxOrdRawValue;
-pub use basic::mapx_raw::MapxRaw;
-pub use basic::single_value::SingleValue;
+pub use basic::orphan::Orphan;
 pub use basic::vecx::Vecx;
 
-pub use versioned::mapx_raw::MapxRawVersioned;
-// pub use versioned::mapx::MapxVersioned;
-// pub use versioned::mapx_ord::MapxOrdVersioned;
-// pub use versioned::vecx::VecxVersioned;
+pub use versioned::mapx::MapxVs;
+pub use versioned::mapx_ord::MapxOrdVs;
+pub use versioned::orphan::OrphanVs;
+pub use versioned::vecx::VecxVs;
 
 pub use common::{
     ende::{KeyEnDe, KeyEnDeOrdered, ValueEnDe},
-    vsdb_flush, vsdb_set_base_dir,
+    vsdb_flush, vsdb_set_base_dir, BranchName, ParentBranchName, VerChecksum,
+    VersionName,
 };
