@@ -1,24 +1,24 @@
 //!
-//! # Versioned functions
+//! # Vs functions
 //!
 //! # Examples
 //!
 //! Used as version-ful:
 //!
 //! ```
-//! use vsdb::MapxRawVersioned;
+//! use vsdb::versioned::mapx_raw::MapxRawVs;
 //!
 //! // TODO
-//! let _l = MapxRawVersioned::new();
+//! let _l = MapxRawVs::new();
 //! ```
 //!
-//! Used as version-less:
+//! Used as version-less(not recommand, use `MapxRaw` instead):
 //!
 //! ```
-//! use vsdb::MapxRawVersioned;
+//! use vsdb::{VersionName, versioned::mapx_raw::MapxRawVs};
 //!
-//! let mut l = MapxRawVersioned::new();
-//! l.version_create(b"test").unwrap();
+//! let mut l = MapxRawVs::new();
+//! l.version_create(VersionName(b"test")).unwrap();
 //!
 //! l.insert(&[1], &[0]);
 //! l.insert(&[1], &[0]);
@@ -41,31 +41,34 @@ mod backend;
 #[cfg(test)]
 mod test;
 
-use crate::common::{BranchID, RawKey, RawValue};
+use crate::common::{
+    BranchName, ParentBranchName, RawKey, RawValue, VerChecksum, VersionName,
+    INITIAL_BRANCH_NAME, NULL,
+};
 use ruc::*;
 use serde::{Deserialize, Serialize};
 use std::ops::RangeBounds;
 
-use backend::{MapxRawVersionedIter, ValueMut, VerSig, INITIAL_BRANCH_NAME, NULL};
+pub(crate) use backend::MapxRawVsIter;
 
 /// Advanced `MapxRaw`, with versioned feature.
-#[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
-pub struct MapxRawVersioned {
-    inner: backend::MapxRawVersioned,
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+pub struct MapxRawVs {
+    inner: backend::MapxRawVs,
 }
 
-impl Default for MapxRawVersioned {
+impl Default for MapxRawVs {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl MapxRawVersioned {
+impl MapxRawVs {
     #[inline(always)]
     #[allow(missing_docs)]
     pub fn new() -> Self {
         Self {
-            inner: backend::MapxRawVersioned::new(),
+            inner: backend::MapxRawVs::new(),
         }
     }
 
@@ -81,8 +84,9 @@ impl MapxRawVersioned {
         &mut self,
         key: &[u8],
         value: &[u8],
-        branch_id: BranchID,
+        branch_name: BranchName,
     ) -> Result<Option<RawValue>> {
+        let branch_id = self.inner.get_branch_id(branch_name).c(d!())?;
         self.inner.insert_by_branch(key, value, branch_id).c(d!())
     }
 
@@ -97,8 +101,9 @@ impl MapxRawVersioned {
     pub fn remove_by_branch(
         &mut self,
         key: &[u8],
-        branch_id: BranchID,
+        branch_name: BranchName,
     ) -> Result<Option<RawValue>> {
+        let branch_id = self.inner.get_branch_id(branch_name).c(d!())?;
         self.inner.remove_by_branch(key, branch_id).c(d!())
     }
 
@@ -110,8 +115,12 @@ impl MapxRawVersioned {
 
     /// Get the value of a key from the head of a specified branch.
     #[inline(always)]
-    pub fn get_by_branch(&self, key: &[u8], branch_name: &[u8]) -> Option<RawValue> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
+    pub fn get_by_branch(
+        &self,
+        key: &[u8],
+        branch_name: BranchName,
+    ) -> Option<RawValue> {
+        let branch_id = self.inner.get_branch_id(branch_name)?;
         self.inner.get_by_branch(key, branch_id)
     }
 
@@ -120,31 +129,14 @@ impl MapxRawVersioned {
     pub fn get_by_branch_version(
         &self,
         key: &[u8],
-        branch_name: &[u8],
-        version_name: &[u8],
+        branch_name: BranchName,
+        version_name: VersionName,
     ) -> Option<RawValue> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id.get(version_name)?;
+        let branch_id = self.inner.get_branch_id(branch_name)?;
+        let version_id = self.inner.get_version_id(branch_name, version_name)?;
         self.inner.get_by_branch_version(key, branch_id, version_id)
     }
 
-    #[inline(always)]
-    #[allow(missing_docs)]
-    pub fn get_mut(&mut self, key: &[u8]) -> Option<ValueMut<'_>> {
-        self.inner.get_mut(key)
-    }
-
-    #[inline(always)]
-    #[allow(missing_docs)]
-    pub fn get_mut_by_branch(
-        &mut self,
-        key: &[u8],
-        branch_name: &[u8],
-    ) -> Option<ValueMut<'_>> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
-        self.inner.get_mut_by_branch(key, branch_id)
-    }
-
     /// Get the value of a key from the default branch,
     /// if the target key does not exist, will try to
     /// search a closest value bigger than the target key.
@@ -160,9 +152,9 @@ impl MapxRawVersioned {
     pub fn get_ge_by_branch(
         &self,
         key: &[u8],
-        branch_name: &[u8],
+        branch_name: BranchName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
+        let branch_id = self.inner.get_branch_id(branch_name)?;
         self.inner.get_ge_by_branch(key, branch_id)
     }
 
@@ -173,11 +165,11 @@ impl MapxRawVersioned {
     pub fn get_ge_by_branch_version(
         &self,
         key: &[u8],
-        branch_name: &[u8],
-        version_name: &[u8],
+        branch_name: BranchName,
+        version_name: VersionName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id.get(version_name)?;
+        let branch_id = self.inner.get_branch_id(branch_name)?;
+        let version_id = self.inner.get_version_id(branch_name, version_name)?;
         self.inner
             .get_ge_by_branch_version(key, branch_id, version_id)
     }
@@ -197,9 +189,9 @@ impl MapxRawVersioned {
     pub fn get_le_by_branch(
         &self,
         key: &[u8],
-        branch_name: &[u8],
+        branch_name: BranchName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
+        let branch_id = self.inner.get_branch_id(branch_name)?;
         self.inner.get_le_by_branch(key, branch_id)
     }
 
@@ -210,29 +202,25 @@ impl MapxRawVersioned {
     pub fn get_le_by_branch_version(
         &self,
         key: &[u8],
-        branch_name: &[u8],
-        version_name: &[u8],
+        branch_name: BranchName,
+        version_name: VersionName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id.get(version_name)?;
+        let branch_id = self.inner.get_branch_id(branch_name)?;
+        let version_id = self.inner.get_version_id(branch_name, version_name)?;
         self.inner
             .get_le_by_branch_version(key, branch_id, version_id)
     }
 
     /// Create an iterator over the default branch.
     #[inline(always)]
-    pub fn iter(&self) -> MapxRawVersionedIter {
+    pub fn iter(&self) -> MapxRawVsIter {
         self.inner.iter()
     }
 
     /// Create an iterator over a specified branch.
     #[inline(always)]
-    pub fn iter_by_branch(&self, branch_name: &[u8]) -> MapxRawVersionedIter {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .unwrap_or(NULL);
+    pub fn iter_by_branch(&self, branch_name: BranchName) -> MapxRawVsIter {
+        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
         self.inner.iter_by_branch(branch_id)
     }
 
@@ -240,73 +228,91 @@ impl MapxRawVersioned {
     #[inline(always)]
     pub fn iter_by_branch_version(
         &self,
-        branch_name: &[u8],
-        version_name: &[u8],
-    ) -> MapxRawVersionedIter {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .unwrap_or(NULL);
-
+        branch_name: BranchName,
+        version_name: VersionName,
+    ) -> MapxRawVsIter {
+        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
         let version_id = self
             .inner
-            .version_name_to_version_id
-            .get(version_name)
+            .get_version_id(branch_name, version_name)
             .unwrap_or(NULL);
-
         self.inner.iter_by_branch_version(branch_id, version_id)
     }
 
     /// Create a range iterator over the default branch.
     #[inline(always)]
-    pub fn range<'a, R: RangeBounds<&'a [u8]>>(
+    pub fn range<'a, R: 'a + RangeBounds<RawKey>>(
         &'a self,
         bounds: R,
-    ) -> MapxRawVersionedIter<'a> {
+    ) -> MapxRawVsIter<'a> {
         self.inner.range(bounds)
     }
 
     /// Create a range iterator over a specified branch.
     #[inline(always)]
-    pub fn range_by_branch<'a, R: RangeBounds<&'a [u8]>>(
+    pub fn range_by_branch<'a, R: 'a + RangeBounds<RawKey>>(
         &'a self,
-        branch_name: &[u8],
+        branch_name: BranchName,
         bounds: R,
-    ) -> MapxRawVersionedIter<'a> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .unwrap_or(NULL);
-
+    ) -> MapxRawVsIter<'a> {
+        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
         self.inner.range_by_branch(branch_id, bounds)
     }
 
     /// Create a range iterator over a specified version of a specified branch.
     #[inline(always)]
-    pub fn range_by_branch_version<'a, R: RangeBounds<&'a [u8]>>(
+    pub fn range_by_branch_version<'a, R: 'a + RangeBounds<RawKey>>(
         &'a self,
-        branch_name: &[u8],
-        version_name: &[u8],
+        branch_name: BranchName,
+        version_name: VersionName,
         bounds: R,
-    ) -> MapxRawVersionedIter<'a> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .unwrap_or(NULL);
-
+    ) -> MapxRawVsIter<'a> {
+        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
         let version_id = self
             .inner
-            .version_name_to_version_id
-            .get(version_name)
+            .get_version_id(branch_name, version_name)
             .unwrap_or(NULL);
-
         self.inner
             .range_by_branch_version(branch_id, version_id, bounds)
     }
 
+    /// Create a range iterator over the default branch.
+    #[inline(always)]
+    pub fn range_ref<'a, R: RangeBounds<&'a [u8]>>(
+        &'a self,
+        bounds: R,
+    ) -> MapxRawVsIter<'a> {
+        self.inner.range_ref(bounds)
+    }
+
+    /// Create a range iterator over a specified branch.
+    #[inline(always)]
+    pub fn range_ref_by_branch<'a, R: RangeBounds<&'a [u8]>>(
+        &'a self,
+        branch_name: BranchName,
+        bounds: R,
+    ) -> MapxRawVsIter<'a> {
+        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
+        self.inner.range_ref_by_branch(branch_id, bounds)
+    }
+
+    /// Create a range iterator over a specified version of a specified branch.
+    #[inline(always)]
+    pub fn range_ref_by_branch_version<'a, R: RangeBounds<&'a [u8]>>(
+        &'a self,
+        branch_name: BranchName,
+        version_name: VersionName,
+        bounds: R,
+    ) -> MapxRawVsIter<'a> {
+        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
+        let version_id = self
+            .inner
+            .get_version_id(branch_name, version_name)
+            .unwrap_or(NULL);
+        self.inner
+            .range_ref_by_branch_version(branch_id, version_id, bounds)
+    }
+
     /// Check if a key exist on the default branch.
     #[inline(always)]
     pub fn contains_key(&self, key: &[u8]) -> bool {
@@ -315,7 +321,7 @@ impl MapxRawVersioned {
 
     /// Check if a key exist on a specified branch.
     #[inline(always)]
-    pub fn contains_key_by_branch(&self, key: &[u8], branch_name: &[u8]) -> bool {
+    pub fn contains_key_by_branch(&self, key: &[u8], branch_name: BranchName) -> bool {
         self.get_by_branch(key, branch_name).is_some()
     }
 
@@ -324,8 +330,8 @@ impl MapxRawVersioned {
     pub fn contains_key_by_branch_version(
         &self,
         key: &[u8],
-        branch_name: &[u8],
-        version_name: &[u8],
+        branch_name: BranchName,
+        version_name: VersionName,
     ) -> bool {
         self.get_by_branch_version(key, branch_name, version_name)
             .is_some()
@@ -339,61 +345,52 @@ impl MapxRawVersioned {
 
     /// Get the total number of items of the head of a specified branch.
     #[inline(always)]
-    pub fn len_by_branch(&self, branch_name: &[u8]) -> usize {
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id.get(branch_name) {
-                id
-            } else {
-                return 0;
-            };
-
-        self.inner.len_by_branch(branch_id)
+    pub fn len_by_branch(&self, branch_name: BranchName) -> usize {
+        self.inner
+            .get_branch_id(branch_name)
+            .map(|id| self.inner.len_by_branch(id))
+            .unwrap_or(0)
     }
 
     /// Get the total number of items of a specified version of a specified branch.
     #[inline(always)]
     pub fn len_by_branch_version(
         &self,
-        branch_name: &[u8],
-        version_name: &[u8],
+        branch_name: BranchName,
+        version_name: VersionName,
     ) -> usize {
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id.get(branch_name) {
-                id
-            } else {
-                return 0;
-            };
-
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id.get(version_name) {
-                id
-            } else {
-                return 0;
-            };
-
-        self.inner.len_by_branch_version(branch_id, version_id)
+        self.inner
+            .get_branch_id(branch_name)
+            .and_then(|br_id| {
+                self.inner
+                    .get_version_id(branch_name, version_name)
+                    .map(|ver_id| self.inner.len_by_branch_version(br_id, ver_id))
+            })
+            .unwrap_or(0)
     }
 
     #[inline(always)]
     #[allow(missing_docs)]
     pub fn is_empty(&self) -> bool {
-        0 == self.len()
+        self.iter().next().is_none()
     }
 
     #[inline(always)]
     #[allow(missing_docs)]
-    pub fn is_empty_by_branch(&self, branch_name: &[u8]) -> bool {
-        0 == self.len_by_branch(branch_name)
+    pub fn is_empty_by_branch(&self, branch_name: BranchName) -> bool {
+        self.iter_by_branch(branch_name).next().is_none()
     }
 
     #[inline(always)]
     #[allow(missing_docs)]
     pub fn is_empty_by_branch_version(
         &self,
-        branch_name: &[u8],
-        version_name: &[u8],
+        branch_name: BranchName,
+        version_name: VersionName,
     ) -> bool {
-        0 == self.len_by_branch_version(branch_name, version_name)
+        self.iter_by_branch_version(branch_name, version_name)
+            .next()
+            .is_none()
     }
 
     /// Clear all data, mainly for testing purpose.
@@ -404,8 +401,8 @@ impl MapxRawVersioned {
 
     /// Create a new version on the default branch.
     #[inline(always)]
-    pub fn version_create(&mut self, version_name: &[u8]) -> Result<()> {
-        self.inner.version_create(version_name).c(d!())
+    pub fn version_create(&mut self, version_name: VersionName) -> Result<()> {
+        self.inner.version_create(version_name.0).c(d!())
     }
 
     /// Create a new version on a specified branch,
@@ -413,85 +410,66 @@ impl MapxRawVersioned {
     #[inline(always)]
     pub fn version_create_by_branch(
         &mut self,
-        version_name: &[u8],
-        branch_name: &[u8],
+        version_name: VersionName,
+        branch_name: BranchName,
     ) -> Result<()> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .c(d!("branch not found"))?;
-
         self.inner
-            .version_create_by_branch(version_name, branch_id)
-            .c(d!())
+            .get_branch_id(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|br_id| {
+                self.inner
+                    .version_create_by_branch(version_name.0, br_id)
+                    .c(d!())
+            })
     }
 
     /// Check if a verison exists on default branch.
     #[inline(always)]
-    pub fn version_exists(&self, version_name: &[u8]) -> bool {
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id.get(version_name) {
-                id
-            } else {
-                return false;
-            };
-
-        self.inner.version_exists(version_id)
+    pub fn version_exists(&self, version_name: VersionName) -> bool {
+        self.inner
+            .get_version_id(BranchName(INITIAL_BRANCH_NAME), version_name)
+            .map(|id| self.inner.version_exists(id))
+            .unwrap_or(false)
     }
 
     /// Check if a version exists on a specified branch(include its parents).
     #[inline(always)]
     pub fn version_exists_on_branch(
         &self,
-        version_name: &[u8],
-        branch_name: &[u8],
+        version_name: VersionName,
+        branch_name: BranchName,
     ) -> bool {
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id.get(version_name) {
-                id
-            } else {
-                return false;
-            };
-
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id.get(branch_name) {
-                id
-            } else {
-                return false;
-            };
-
-        self.inner.version_exists_on_branch(version_id, branch_id)
+        self.inner
+            .get_branch_id(branch_name)
+            .and_then(|br_id| {
+                self.inner
+                    .get_version_id(branch_name, version_name)
+                    .map(|ver_id| self.inner.version_exists_on_branch(ver_id, br_id))
+            })
+            .unwrap_or(false)
     }
 
     /// Check if a version is directly created on the default branch.
     #[inline(always)]
-    pub fn version_created(&self, version_name: &[u8]) -> bool {
-        self.version_created_on_branch(version_name, INITIAL_BRANCH_NAME)
+    pub fn version_created(&self, version_name: VersionName) -> bool {
+        self.version_created_on_branch(version_name, BranchName(INITIAL_BRANCH_NAME))
     }
 
     /// Check if a version is directly created on a specified branch(exclude its parents).
     #[inline(always)]
     pub fn version_created_on_branch(
         &self,
-        version_name: &[u8],
-        branch_name: &[u8],
+        version_name: VersionName,
+        branch_name: BranchName,
     ) -> bool {
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id.get(version_name) {
-                id
-            } else {
-                return false;
-            };
-
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id.get(branch_name) {
-                id
-            } else {
-                return false;
-            };
-
-        self.inner.version_created_on_branch(version_id, branch_id)
+        self.inner
+            .get_branch_id(branch_name)
+            .and_then(|br_id| {
+                self.inner
+                    .get_version_id(branch_name, version_name)
+                    .map(|ver_id| self.inner.version_created_on_branch(ver_id, br_id))
+            })
+            .unwrap_or(false)
     }
 
     /// Remove the newest version on the default branch.
@@ -514,48 +492,43 @@ impl MapxRawVersioned {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn version_pop_by_branch(&mut self, branch_name: &[u8]) -> Result<()> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .c(d!("branch not found"))?;
-
-        self.inner.version_pop_by_branch(branch_id).c(d!())
+    pub fn version_pop_by_branch(&mut self, branch_name: BranchName) -> Result<()> {
+        self.inner
+            .get_branch_id(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|br_id| self.inner.version_pop_by_branch(br_id).c(d!()))
     }
 
     /// Create a new branch based on the head of the default branch.
     #[inline(always)]
-    pub fn branch_create(&mut self, branch_name: &[u8]) -> Result<()> {
-        self.inner.branch_create(branch_name).c(d!())
+    pub fn branch_create(&mut self, branch_name: BranchName) -> Result<()> {
+        self.inner.branch_create(branch_name.0).c(d!())
     }
 
     /// Create a new branch based on the head of a specified branch.
     #[inline(always)]
     pub fn branch_create_by_base_branch(
         &mut self,
-        branch_name: &[u8],
-        base_branch_name: &[u8],
+        branch_name: BranchName,
+        base_branch_name: ParentBranchName,
     ) -> Result<()> {
-        let base_branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(base_branch_name)
-            .c(d!("base branch not found"))?;
-
         self.inner
-            .branch_create_by_base_branch(branch_name, base_branch_id)
-            .c(d!())
+            .get_branch_id(BranchName(base_branch_name.0))
+            .c(d!("base branch not found"))
+            .and_then(|base_br_id| {
+                self.inner
+                    .branch_create_by_base_branch(branch_name.0, base_br_id)
+                    .c(d!())
+            })
     }
 
     /// Check if a branch exists or not.
     #[inline(always)]
-    pub fn branch_exists(&self, branch_name: &[u8]) -> bool {
-        if let Some(branch_id) = self.inner.branch_name_to_branch_id.get(branch_name) {
-            self.inner.branch_exists(branch_id)
-        } else {
-            false
-        }
+    pub fn branch_exists(&self, branch_name: BranchName) -> bool {
+        self.inner
+            .get_branch_id(branch_name)
+            .map(|id| self.inner.branch_exists(id))
+            .unwrap_or(false)
     }
 
     /// Remove a branch, remove all changes directly made by this branch.
@@ -566,8 +539,8 @@ impl MapxRawVersioned {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn branch_remove(&mut self, branch_name: &[u8]) -> Result<()> {
-        if let Some(branch_id) = self.inner.branch_name_to_branch_id.get(branch_name) {
+    pub fn branch_remove(&mut self, branch_name: BranchName) -> Result<()> {
+        if let Some(branch_id) = self.inner.get_branch_id(branch_name) {
             self.inner.branch_remove(branch_id).c(d!())
         } else {
             Err(eg!("branch not found"))
@@ -582,13 +555,11 @@ impl MapxRawVersioned {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn branch_truncate(&mut self, branch_name: &[u8]) -> Result<()> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .c(d!("branch not found"))?;
-        self.inner.branch_truncate(branch_id).c(d!())
+    pub fn branch_truncate(&mut self, branch_name: BranchName) -> Result<()> {
+        self.inner
+            .get_branch_id(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|br_id| self.inner.branch_truncate(br_id).c(d!()))
     }
 
     /// Remove all changes directly made by versions(bigger than `last_version_id`) of this branch.
@@ -601,22 +572,20 @@ impl MapxRawVersioned {
     #[inline(always)]
     pub fn branch_truncate_to(
         &mut self,
-        branch_name: &[u8],
-        last_version_name: &[u8],
+        branch_name: BranchName,
+        last_version_name: VersionName,
     ) -> Result<()> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .c(d!("branch not found"))?;
-        let last_version_id = self
-            .inner
-            .version_name_to_version_id
-            .get(last_version_name)
-            .c(d!("version not found"))?;
         self.inner
-            .branch_truncate_to(branch_id, last_version_id)
-            .c(d!())
+            .get_branch_id(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|br_id| {
+                self.inner
+                    .get_version_id(branch_name, last_version_name)
+                    .c(d!("version not found"))
+                    .and_then(|last_ver_id| {
+                        self.inner.branch_truncate_to(br_id, last_ver_id).c(d!())
+                    })
+            })
     }
 
     /// Remove the newest version on a specified branch.
@@ -627,82 +596,89 @@ impl MapxRawVersioned {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn branch_pop_version(&mut self, branch_name: &[u8]) -> Result<()> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .c(d!("branch not found"))?;
-        self.inner.branch_pop_version(branch_id).c(d!())
+    pub fn branch_pop_version(&mut self, branch_name: BranchName) -> Result<()> {
+        self.inner
+            .get_branch_id(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|id| self.inner.branch_pop_version(id).c(d!()))
     }
 
     /// Merge a branch to its parent branch.
     #[inline(always)]
-    pub fn branch_merge_to_parent(&mut self, branch_name: &[u8]) -> Result<()> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .c(d!("branch not found"))?;
-        self.inner.branch_merge_to_parent(branch_id).c(d!())
+    pub fn branch_merge_to_parent(&mut self, branch_name: BranchName) -> Result<()> {
+        self.inner
+            .get_branch_id(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|id| self.inner.branch_merge_to_parent(id).c(d!()))
     }
 
     /// Check if a branch has children branches.
     #[inline(always)]
-    pub fn branch_has_children(&self, branch_name: &[u8]) -> bool {
-        if let Some(id) = self.inner.branch_name_to_branch_id.get(branch_name) {
-            self.inner.branch_has_children(id)
-        } else {
-            false
-        }
+    pub fn branch_has_children(&self, branch_name: BranchName) -> bool {
+        self.inner
+            .get_branch_id(branch_name)
+            .map(|id| self.inner.branch_has_children(id))
+            .unwrap_or(false)
+    }
+
+    /// Make a branch to be default,
+    /// all default operations will be applied to it.
+    #[inline(always)]
+    pub fn branch_set_default(&mut self, branch_name: BranchName) -> Result<()> {
+        self.inner
+            .get_branch_id(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|brid| self.inner.branch_set_default(brid).c(d!()))
     }
 
     /// Get the signature of the head of the default branch.
     #[inline(always)]
-    pub fn sig_get(&self) -> Option<VerSig> {
-        self.inner.sig_get()
+    pub fn checksum_get(&self) -> Option<VerChecksum> {
+        self.inner.checksum_get()
     }
 
     /// Get the signature of the head of a specified branch.
     #[inline(always)]
-    pub fn sig_get_by_branch(&self, branch_name: &[u8]) -> Option<VerSig> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
-        self.inner.sig_get_by_branch(branch_id)
+    pub fn checksum_get_by_branch(
+        &self,
+        branch_name: BranchName,
+    ) -> Option<VerChecksum> {
+        self.inner
+            .get_branch_id(branch_name)
+            .and_then(|id| self.inner.checksum_get_by_branch(id))
     }
 
     /// Get the signature of a specified version of a specified branch.
     #[inline(always)]
-    pub fn sig_get_by_branch_version(
+    pub fn checksum_get_by_branch_version(
         &self,
-        branch_name: &[u8],
-        version_name: &[u8],
-    ) -> Option<VerSig> {
-        let branch_id = self.inner.branch_name_to_branch_id.get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id.get(version_name)?;
+        branch_name: BranchName,
+        version_name: VersionName,
+    ) -> Option<VerChecksum> {
+        let br_id = self.inner.get_branch_id(branch_name)?;
+        let ver_id = self.inner.get_version_id(branch_name, version_name)?;
         self.inner
-            .sig_get_by_branch_version(branch_id, Some(version_id))
+            .checksum_get_by_branch_version(br_id, Some(ver_id))
     }
 
     /// Clean outdated versions out of the default reserved number.
     #[inline(always)]
-    pub fn prune(&mut self) -> Result<()> {
-        self.inner.prune().c(d!())
+    pub fn prune(&mut self, reserved_ver_num: Option<usize>) -> Result<()> {
+        self.inner.prune(reserved_ver_num).c(d!())
     }
 
     /// Clean outdated versions out of a specified reserved number.
     #[inline(always)]
     pub fn prune_by_branch(
         &mut self,
-        branch_name: &[u8],
-        reserved_ver_num: usize,
+        branch_name: BranchName,
+        reserved_ver_num: Option<usize>,
     ) -> Result<()> {
-        let branch_id = self
-            .inner
-            .branch_name_to_branch_id
-            .get(branch_name)
-            .c(d!())?;
         self.inner
-            .prune_by_branch(branch_id, reserved_ver_num)
+            .get_branch_id(branch_name)
             .c(d!())
+            .and_then(|br_id| {
+                self.inner.prune_by_branch(br_id, reserved_ver_num).c(d!())
+            })
     }
 }
